/**
 * EasyBeans
 * Copyright (C) 2007-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EarExploded.java 2530 2008-02-27 17:58:05Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.ant.archive.exploded;

import java.io.File;
import java.util.List;

import org.apache.tools.ant.Project;
import org.ow2.easybeans.ant.archive.AbsArchive;
import org.ow2.easybeans.ant.archive.Ejb;
import org.ow2.easybeans.ant.archive.War;
import org.ow2.easybeans.ant.archive.api.IEar;
import org.ow2.easybeans.ant.archive.info.EarInfo;

/**
 * Creates an EAR exploded archive.
 * @author Florent Benoit
 */
public class EarExploded extends AbsExplodedArchive implements IEar {

    /**
     * Path to the Standard deployment descriptor.
     */
    private static final String DEPLOYMENT_DESCRIPTOR = "META-INF/application.xml";

    /**
     * Path to the Specific deployment descriptor.
     */
    private static final String SPECIFIC_DEPLOYMENT_DESCRIPTOR = "META-INF/easybeans.xml";

    /**
     * Ear info object.
     */
    private EarInfo earInfo = null;


    /**
     * Creates an archive for the given project.
     * @param p the given project
     */
    public EarExploded(final Project p) {
        super(p);
    }

    /**
     * Sets the information about an EAR archive.
     * @param earInfo the object that holds data information.
     */
    public void setEarInfo(final EarInfo earInfo) {
        setArchiveInfo(earInfo);
        this.earInfo = earInfo;
    }

    /**
     * Gets the path to the standard deployment descriptor.
     * @return the path to the standard deployment descriptor.
     */
    @Override
    public String getDDStandardName() {
        return DEPLOYMENT_DESCRIPTOR;
    }

    /**
     * Execute the task.
     */
    @Override
    public void execute() {
        // create directory
        earInfo.getDest().mkdirs();

        // Include the DD if any and initial fileset
        super.execute();

        // Execute children by changing the destination directory
        List<Ejb> ejbs = earInfo.getEjbs();
        List<War> wars = earInfo.getWars();

        if (ejbs != null) {
            for (Ejb ejb : ejbs) {
                updateArchive(ejb);
                ejb.execute();
            }
        }

        if (wars != null) {
            for (War war : wars) {
                updateArchive(war);
                war.execute();
            }
        }
    }

    /**
     * Update the archive with some settings.
     * @param archive the archive to update
     */
    void updateArchive(final AbsArchive archive) {
        archive.setDest(new File(earInfo.getDest(), archive.getDest().getName()));
    }

    /**
     * Gets the path to the specific deployment descriptor.
     * @return the path to the specific deployment descriptor.
     */
    @Override
    public String getDDSpecificame() {
        return SPECIFIC_DEPLOYMENT_DESCRIPTOR;
    }
}
