/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClientFile.java 2530 2008-02-27 17:58:05Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.ant.archive.file;

import java.io.File;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Jar;
import org.apache.tools.ant.taskdefs.Manifest;
import org.apache.tools.ant.taskdefs.ManifestException;
import org.apache.tools.ant.taskdefs.Manifest.Attribute;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.ZipFileSet;
import org.ow2.easybeans.ant.archive.api.IClient;
import org.ow2.easybeans.ant.archive.info.ArchiveInfo;
import org.ow2.easybeans.ant.archive.info.ClientInfo;

/**
 * Creates a Client file.
 * @author Florent Benoit
 */
public class ClientFile extends Jar implements IClient {

    /**
     * Path to the Standard deployment descriptor.
     */
    private static final String DEPLOYMENT_DESCRIPTOR = "META-INF/application-client.xml";

    /**
     * Reference to the client archive info object.
     */
    private ClientInfo clientInfo = null;

    /**
     * Reference to the archive info object.
     */
    private ArchiveInfo archiveInfo = null;

    /**
     * Creates an archive for the given project.
     * @param p the given project
     */
    public ClientFile(final Project p) {
        super();
        setProject(p);
    }

    /**
     * Sets the information about an archive.
     * @param archiveInfo the object that holds data information.
     */
    public void setArchiveInfo(final ArchiveInfo archiveInfo) {
        this.archiveInfo = archiveInfo;
    }

    /**
     * Sets the information about a Client archive.
     * @param clientInfo the object that holds data information.
     */
    public void setClientInfo(final ClientInfo clientInfo) {
        setArchiveInfo(clientInfo);
        this.clientInfo = clientInfo;
    }

    /**
     * Execute the task.
     */
    @Override
    public void execute() {

        // Deployment descriptor
        if (archiveInfo.getDD() != null) {
            setDD(archiveInfo.getDD());
        }

        // Main class attribute ?
        String mainClass = clientInfo.getMainClass();
        if (mainClass != null) {
            Manifest mf = new Manifest();
            Attribute attribute = new Attribute();
            attribute.setName("Main-Class");
            attribute.setValue(mainClass);

            // Add attribute
            try {
                mf.addConfiguredAttribute(attribute);
            } catch (ManifestException e) {
                throw new BuildException("Cannot add the Main-Class attribute in the manifest", e);
            }

            // Add Manifest
            try {
                addConfiguredManifest(mf);
            } catch (ManifestException e) {
                throw new BuildException("Cannot add the manifest", e);
            }
        }


        // dest file
        setDestFile(archiveInfo.getDest());

        // fileset
        for (FileSet fileSet : archiveInfo.getFileSetList()) {
            addFileset(fileSet);
        }

        super.execute();
    }

    /**
     * Add the given DD file into the archive.
     * @param dd the path to the DDesc file.
     */
    public void setDD(final File dd) {
        ZipFileSet zipFileSet = new ZipFileSet();
        zipFileSet.setFile(dd);
        zipFileSet.setFullpath(DEPLOYMENT_DESCRIPTOR);
        addFileset(zipFileSet);
    }

}
