/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EarFile.java 2530 2008-02-27 17:58:05Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.ant.archive.file;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Jar;
import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.types.ZipFileSet;
import org.ow2.easybeans.ant.archive.AbsArchive;
import org.ow2.easybeans.ant.archive.Ejb;
import org.ow2.easybeans.ant.archive.War;
import org.ow2.easybeans.ant.archive.api.IEar;
import org.ow2.easybeans.ant.archive.info.ArchiveInfo;
import org.ow2.easybeans.ant.archive.info.EarInfo;

/**
 * Creates an EAR file.
 * @author Florent Benoit
 */
public class EarFile extends Jar implements IEar {

    /**
     * Path to the Standard deployment descriptor.
     */
    private static final String DEPLOYMENT_DESCRIPTOR = "META-INF/application.xml";

    /**
     * Ear info object.
     */
    private EarInfo earInfo = null;

    /**
     * Creates an archive for the given project.
     * @param p the given project
     */
    public EarFile(final Project p) {
        super();
        setProject(p);
    }

    /**
     * Sets the information about an EAR archive.
     * @param earInfo the object that holds data information.
     */
    public void setEarInfo(final EarInfo earInfo) {
        this.earInfo = earInfo;
    }

    /**
     * Execute the task.
     */
    @Override
    public void execute() {

        // First, package each ejb, war, etc. (sub modules)
        // Execute children by changing the destination directory
        List<Ejb> ejbs = earInfo.getEjbs();
        List<War> wars = earInfo.getWars();

        // Add all archives
        List<AbsArchive> archives = new ArrayList<AbsArchive>();
        if (ejbs != null) {
            for (Ejb ejb : ejbs) {
                archives.add(ejb);
            }
        }
        if (wars != null) {
            for (War war : wars) {
                archives.add(war);
            }
        }

        // Update archives
        for (AbsArchive archive : archives) {
            // Update path
            updateArchive(archive);

            // Build the file
            archive.execute();

            // Now, add each archive built in the fileset of the Ear
            addArchiveInFileSet(archive);
        }

        // Deployment descriptor
        if (earInfo.getDD() != null) {
            setDD(earInfo.getDD());
        }

        // dest file
        setDestFile(earInfo.getDest());

        // fileset
        for (FileSet fileSet : earInfo.getFileSetList()) {
            addFileset(fileSet);
        }

        // Create directory if it is not existing
        File earFileParentDirectory = earInfo.getDest().getParentFile();
        if (!earFileParentDirectory.exists()) {
            log("Creating directory '" + earFileParentDirectory + "'", Project.MSG_INFO);
            earFileParentDirectory.mkdirs();
        }

        super.execute();

        // Clean all temporary files
        for (AbsArchive archive : archives) {
            File tmpFile = archive.getDest();
            if (tmpFile.exists()) {
                tmpFile.delete();
            }
        }

    }

    /**
     * Update settings of the given archive.
     * @param archive the archive to update.
     */
    void updateArchive(final AbsArchive archive) {
        // change exploded mode to false if ear level is false
        if (archive.isExploded() && !earInfo.isExploded()) {
            log("Changing exploded mode to false as the EAR is not in exploded mode.", Project.MSG_INFO);
            archive.setExploded(false);
        }

        // name is defined ? if not set it to the last name of the dest
        if (archive.getName() == null) {
            archive.setName(archive.getDest().getName());
        }

        try {
            archive.setDest(File.createTempFile("easybeans-ant", ".tmp"));
        } catch (IOException e) {
            throw new BuildException("Cannot create a temp file", e);
        }
        // Delete the temporary file, it should be created by the archive
        // packaging
        archive.getDest().delete();
    }

    /**
     * Add the given DD file into the archive.
     * @param dd the path to the DDesc file.
     */
    public void setDD(final File dd) {
        ZipFileSet zipFileSet = new ZipFileSet();
        zipFileSet.setFile(dd);
        zipFileSet.setFullpath(DEPLOYMENT_DESCRIPTOR);
        addFileset(zipFileSet);
    }
    /**
     * Add the given archive in a fileset.
     * @param archive the path to the file to include.
     */
    public void addArchiveInFileSet(final AbsArchive archive) {
        ZipFileSet zipFileSet = new ZipFileSet();
        zipFileSet.setFile(archive.getDest());
        zipFileSet.setFullpath(archive.getName());
        addFileset(zipFileSet);
    }

    /**
     * Sets the information about an archive.
     * @param archiveInfo the object that holds data information.
     */
    public void setArchiveInfo(final ArchiveInfo archiveInfo) {
        // nothing
    }

}
