/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:CarolComponent.java 1477 2007-06-16 16:50:19Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.component.carol;

import static org.ow2.easybeans.util.url.URLUtils.fileToURL;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;

import javax.naming.Context;

import org.objectweb.carol.jndi.ns.NameServiceException;
import org.objectweb.carol.jndi.ns.NameServiceManager;
import org.objectweb.carol.rmi.jrmp.interceptor.JInterceptorStore;
import org.objectweb.carol.util.configuration.ConfigurationException;
import org.objectweb.carol.util.configuration.ConfigurationRepository;
import org.ow2.easybeans.component.api.EZBComponentException;
import org.ow2.easybeans.component.itf.RegistryComponent;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This component allows to start a registry.
 * @author Florent Benoit
 */
public class CarolComponent implements RegistryComponent {

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(CarolComponent.class);

    /**
     * Default prefix protocol.
     */
    private static final String DEFAULT_PREFIX_PROTOCOL = "rmi";

    /**
     * List of protocols.
     */
    private List<Protocol> protocols = null;

    /**
     * Global Initial Context Factory.
     */
    private String initialContextFactory = null;

    /**
     * Creates a new Carol component.
     */
    public CarolComponent() {
        protocols = new ArrayList<Protocol>();
    }

    /**
     * Init method.<br/>
     * This method is called before the start method.
     * @throws EZBComponentException if the initialization has failed.
     */
    public void init() throws EZBComponentException {
        // check
        if (protocols == null || protocols.isEmpty()) {
            logger.debug("No protocols, use the existing carol configuration");
            return;
        }

        // Get Value of the Initial Factory
        initialContextFactory = System.getProperty(Context.INITIAL_CONTEXT_FACTORY);

        // List of protocols
        String lstProtocol = null;

        // Create configuration
        StringBuilder carolConf = new StringBuilder();
        for (Protocol protocol : protocols) {
            // Build list of protocols
            if (lstProtocol != null) {
                lstProtocol += ",";
                lstProtocol += protocol.getName();
            } else {
                lstProtocol = protocol.getName();
            }


            //Define URL
            carolConf.append("carol.");
            carolConf.append(protocol.getName());
            carolConf.append(".url=");
            if (protocol.getUrl() != null) {
                // set defined URL
                carolConf.append(protocol.getUrl());
            } else {
                // compute URL
                String host = protocol.getHostname();
                int portNumber = protocol.getPortNumber();
                carolConf.append(DEFAULT_PREFIX_PROTOCOL);
                carolConf.append("://");
                carolConf.append(host);
                carolConf.append(":");
                carolConf.append(portNumber);
            }
            carolConf.append("\n");

        }

        // Set carol mode in server mode (will use fixed port if set)
        System.setProperty("carol.server.mode", "true");
        try {
            // hack
            Writer fw;
            File fConf = new File(System.getProperty("java.io.tmpdir") + File.separator + System.getProperty("user.name")
                    + "ejb3-carol.properties");
            try {
                fw = new FileWriter(fConf);
                fw.write("carol.protocols=");
                fw.write(lstProtocol);
                fw.write("\n");
                fw.write(carolConf.toString());
                fw.write("carol.jvm.rmi.local.registry=true");
                fw.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
            ConfigurationRepository.init(fileToURL(fConf));
        } catch (ConfigurationException e) {
            throw new EZBComponentException("Cannot initialize registry", e);
        }

        try {
            ConfigurationRepository.addInterceptors("jrmp", "org.objectweb.jotm.jta.rmi.JTAInterceptorInitializer");
        } catch (ConfigurationException e) {
            throw new EZBComponentException("Cannot add JOTM interceptors", e);
        }

        try {
            ConfigurationRepository.addInterceptors("iiop", "org.objectweb.jotm.ots.OTSORBInitializer");
        } catch (ConfigurationException e) {
            throw new EZBComponentException("Cannot add JOTM interceptors", e);
        }

        try {
            ConfigurationRepository.addInterceptors("jrmp",
                    "org.ow2.easybeans.security.propagation.rmi.jrmp.interceptors.SecurityInitializer");
        } catch (ConfigurationException e) {
            throw new EZBComponentException("Cannot add Security interceptors", e);
        }

        // Init Initializers
        JInterceptorStore.getJRMPInitializers();

    }


    /**
     * Start method.<br/>
     * This method is called after the init method.
     * @throws EZBComponentException if the start has failed.
     */
    public void start() throws EZBComponentException {
        // Start registry only if there are protocols
        if (protocols != null && !protocols.isEmpty()) {
            try {
                NameServiceManager.startNS();
            } catch (NameServiceException e) {
                throw new EZBComponentException("Cannot start registry", e);
            }
        }
    }


    /**
     * Stop method.<br/>
     * This method is called when component needs to be stopped.
     * @throws EZBComponentException if the stop is failing.
     */
    public void stop() throws EZBComponentException {
        // Stop registry only if there are protocols
        if (protocols != null && !protocols.isEmpty()) {
            try {
                NameServiceManager.stopNS();
            } catch (NameServiceException e) {
                throw new EZBComponentException("Cannot stop the registry", e);
            }

            // Restore previous value of the Initial Context Factory
            if (initialContextFactory != null) {
                System.setProperty(Context.INITIAL_CONTEXT_FACTORY, initialContextFactory);
            } else {
                // Remove property
                System.getProperties().remove(Context.INITIAL_CONTEXT_FACTORY);
            }

        }
    }

    /**
     * Gets the protocols defined for the start.
     * @return the list of protocols.
     */
    public List<Protocol> getProtocols() {
        return protocols;
    }

    /**
     * Sets the list of protocols.
     * @param protocols the list of protocols configured for this server.
     */
    public void setProtocols(final List<Protocol> protocols) {
        this.protocols = protocols;
    }

    /**
     * Gets the default Provider URL.
     * @return the provider URL that is used by default.
     */
    public String getProviderURL() {
        return ConfigurationRepository.getCurrentConfiguration().getProviderURL();
    }

}
