/**
 * EasyBeans
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: StatelessSessionFactory.java 2233 2008-01-16 17:51:35Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.container.session.stateless;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.ejb.ApplicationException;
import javax.ejb.EJBException;
import javax.ejb.NoSuchEJBException;
import javax.ejb.Timer;

import org.ow2.easybeans.api.EZBContainer;
import org.ow2.easybeans.api.FactoryException;
import org.ow2.easybeans.api.bean.EasyBeansSLSB;
import org.ow2.easybeans.container.session.SessionFactory;
import org.ow2.easybeans.rpc.JEJBResponse;
import org.ow2.easybeans.rpc.api.EJBResponse;
import org.ow2.easybeans.rpc.api.RPCException;
import org.ow2.util.pool.api.PoolException;
import org.ow2.util.pool.impl.JPool;
import org.ow2.util.pool.impl.PoolEntryStatistics;
import org.ow2.util.pool.impl.PoolFactory;

/**
 * This class manages the stateless session bean and its creation/lifecycle.
 * @author Florent Benoit
 */
public class StatelessSessionFactory extends SessionFactory<EasyBeansSLSB> implements
        PoolFactory<EasyBeansSLSB, Long> {

    /**
     * Builds a new factory with a given name and its container.
     * @param className name of this factory (name of class that is managed)
     * @param container the root component of this factory.
     * @throws FactoryException if class can't be loaded.
     */
    public StatelessSessionFactory(final String className, final EZBContainer container) throws FactoryException {
        super(className, container);
        setPool(new JPool<EasyBeansSLSB, Long>(this));
    }

    /**
     * Checks if the given object with the given clue is matching.
     * @param object given object against which the check should be done.
     * @param clue the object used as clue to check the matching.
     * @return true if it is matching, else false.
     */
    public boolean isMatching(final EasyBeansSLSB object, final Long clue) {
        // all instances are matching.
        // But we shouldn't go here
        return true;
    }

    /**
     * Validate an instance by giving some statistics.
     * @param object the instance to validate
     * @param stats some statistics to help in the validating process.
     * @return true if the element is valid, else false.
     */
    public boolean validate(final EasyBeansSLSB object, final PoolEntryStatistics stats) {
        return true;
    }

    /**
     * Gets a new ID or a null value.
     * @param beanId given id.
     * @return new id
     */
    @Override
    protected Long getId(final Long beanId) {
        return null;
    }

    /**
     * Gets a bean for the given id.
     * @param beanId id of the expected bean.
     * @return a Stateless bean.
     * @throws IllegalArgumentException if bean is not found.
     */
    @Override
    protected EasyBeansSLSB getBean(final Long beanId) throws IllegalArgumentException {
        try {
            return getPool().get();
        } catch (PoolException e) {
            throw new IllegalArgumentException("Cannot get element in the pool", e);
        }
    }

    /**
     * Do a local call on a method of this factory.
     * @param hash the hash of the method to execute.
     * @param methodArgs the arguments of the method
     * @param beanId the id of the bean that we want (stateful).
     * @return response container new id (if any) and value.
     */
    @Override
    public EJBResponse localCall(final long hash, final Object[] methodArgs, final Long beanId) {

        // build EJB Response and set the id
        EJBResponse ejbResponse = new JEJBResponse();

        EasyBeansSLSB bean = null;
        try {
            bean = getBean(null);
        } catch (IllegalArgumentException e) {
            ejbResponse.setRPCException(new RPCException("Cannot get element in the pool", e));
            return ejbResponse;
        } catch (NoSuchEJBException e) {
            ejbResponse.setRPCException(new RPCException("Bean has been removed", e));
            return ejbResponse;
        }

        Method m = getHashes().get(Long.valueOf(hash));

        if (m == null) {
            ejbResponse.setRPCException(new RPCException("Cannot find method called on the bean '" + getClassName() + "'."));
            return ejbResponse;
        }

        Object value = null;

        // set ClassLoader
        ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getContainer().getClassLoader());

        try {
            value = m.invoke(bean, methodArgs);
        } catch (IllegalArgumentException e) {
            ejbResponse.setRPCException(new RPCException(e));
        } catch (IllegalAccessException e) {
            ejbResponse.setRPCException(new RPCException(e));
        } catch (InvocationTargetException e) {
            Throwable cause = e.getCause();
            RPCException rpcException = new RPCException(cause);
            // ApplicationException ?
            ApplicationException applicationException = getBeanInfo().getApplicationExceptions().get(cause.getClass().getName());
            if (applicationException != null) {
                rpcException.setApplicationException();
            }
            ejbResponse.setRPCException(rpcException);
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
            // push back into the pool
            try {
                getPool().release(bean);
            } catch (PoolException e) {
                ejbResponse.setRPCException(new RPCException("cannot release bean", e));
            }

        }
        ejbResponse.setValue(value);
        return ejbResponse;

    }

    /**
     * Callback called when object is gonna be removed.
     * @param instance that is being removed from the pool.
     */
    @Override
    public void remove(final EasyBeansSLSB instance) {
        super.remove(instance);
        instance.setEasyBeansRemoved(true);
    }


    /**
     * Notified when the timer service send a Timer object.
     * It has to call the Timed method.
     * @param timer the given timer object that will be given to the timer method.
     */
    public void notifyTimeout(final Timer timer) {
        // Call the EasyBeans timer method on a given bean instance
        EasyBeansSLSB bean = null;
        bean = getBean(null);

        //set ClassLoader
        ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getContainer().getClassLoader());

        // Call the timer method on the bean
        try {
            bean.timeoutCallByEasyBeans(timer);
        } finally {
            // Reset classloader
            Thread.currentThread().setContextClassLoader(oldClassLoader);

            // push back into the pool
            try {
                getPool().release(bean);
            } catch (PoolException e) {
                throw new EJBException("cannot release bean", e);
            }
        }
    }

}
