/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClassesEnhancer.java 2447 2008-02-14 17:09:49Z gaellalire $
 * --------------------------------------------------------------------------
 */
package org.ow2.easybeans.tests.enhancer;

import java.net.URL;
import java.net.URLClassLoader;
import java.util.List;

import org.ow2.easybeans.deployment.annotations.analyzer.configurator.metadata.ejbjar.EjbJarArchiveMetadataConfigurator;
import org.ow2.easybeans.deployment.annotations.helper.ResolverHelper;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarArchiveMetadata;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarClassMetadata;
import org.ow2.easybeans.deployment.metadata.ejbjar.EjbJarMethodMetadata;
import org.ow2.easybeans.enhancer.Enhancer;
import org.ow2.util.deployment.annotations.analyzer.ArchiveScanner;


/**
 * Enhance a set of given classes.
 * Rely on core enhancer.
 * @author Florent Benoit
 */
public final class ClassesEnhancer extends Enhancer {

    /**
     * Type available (for adding adapter).
     */
    public static enum TYPE {INTERCEPTOR, CALLBACK, ALL}

    /**
     * Class extension.
     */
    public static final String EXT_CLASS = ".class";

    /**
     * Creates an new enhancer.
     * @param loader classloader where to define enhanced classes.
     * @param ejbJarAnnotationMetadata object with references to the metadata.
     */
    public ClassesEnhancer(final ClassLoader loader, final EjbJarArchiveMetadata ejbJarAnnotationMetadata) {
        super(loader, ejbJarAnnotationMetadata, null);
    }

    /**
     * Enhance the classes simulating a Bean. It uses a child classloder of the current thread.<br>
     * @param classesToEnhance the list of class on which run class adapters
     * @param type the type of adapter to run.
     * @throws Exception if it fails
     */
    public static void enhanceNewClassLoader(final List<String> classesToEnhance, final TYPE type) throws Exception {
        ClassLoader loader = Thread.currentThread().getContextClassLoader();
        ClassLoader childLoader = new URLClassLoader(new URL[]{}, loader);
        Thread.currentThread().setContextClassLoader(childLoader);
        try{
            enhance(classesToEnhance, TYPE.INTERCEPTOR);
        } finally{
            Thread.currentThread().setContextClassLoader(loader);
        }
    }

    /**
     * Enhance the classes simulating a Bean.<br>
     * @param classesToEnhance the list of class on which run class adapters
     * @param type the type of adapter to run.
     * @throws Exception if it fails
     */
    public static void enhance(final List<String> classesToEnhance, final TYPE type) throws Exception {

        ClassLoader loader = Thread.currentThread().getContextClassLoader();

        ArchiveScanner archiveScanner = new ArchiveScanner(new ArchiveInMemory(loader, classesToEnhance));

        EjbJarArchiveMetadataConfigurator ejbJarArchiveMetadataConfigurator = new EjbJarArchiveMetadataConfigurator();

        archiveScanner.addArchiveConfigurator(ejbJarArchiveMetadataConfigurator);

        archiveScanner.scanAll();

        EjbJarArchiveMetadata ejbJarAnnotationMetadata = ejbJarArchiveMetadataConfigurator.getEjbJarArchiveMetadata();

        ResolverHelper.resolve(ejbJarAnnotationMetadata);

        // Remove some TX interceptors (so it works offline)
        for (EjbJarClassMetadata classAnnotationMetadata : ejbJarAnnotationMetadata
                .getEjbJarClassMetadataCollection()) {
            if (classAnnotationMetadata.isBean()) {
                // Remove global EasyBeans interceptors
                classAnnotationMetadata.setGlobalEasyBeansInterceptors(null);
                for (EjbJarMethodMetadata m : classAnnotationMetadata.getMethodMetadataCollection()) {
                    m.setInterceptors(null);
                }
            }
        }

        ClassesEnhancer classesEnhancer = new ClassesEnhancer(loader, ejbJarAnnotationMetadata);
        classesEnhancer.enhance();
    }

}
