/**
 * EasyBeans
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EjbJarMethodMetadata.java 2372 2008-02-08 18:18:37Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.metadata.ejbjar;

import static org.ow2.easybeans.deployment.metadata.CommonMetadata.concatStringBuilder;

import java.util.List;
import java.util.Map;

import javax.ejb.Remove;
import javax.ejb.TransactionAttributeType;

import org.ow2.easybeans.deployment.annotations.InterceptorType;
import org.ow2.easybeans.deployment.annotations.JClassInterceptor;
import org.ow2.easybeans.deployment.annotations.TransactionAttributeLevel;
import org.ow2.easybeans.deployment.annotations.impl.JInterceptors;
import org.ow2.easybeans.deployment.metadata.CommonMethodMetadata;
import org.ow2.easybeans.deployment.metadata.interfaces.IAnnotationSecurityPermitAll;
import org.ow2.easybeans.deployment.metadata.interfaces.IAnnotationSecurityRolesAllowed;
import org.ow2.easybeans.deployment.metadata.interfaces.IEJBInterceptors;
import org.ow2.easybeans.deployment.metadata.interfaces.IInterceptorExcludeDefaultInterceptors;
import org.ow2.easybeans.deployment.metadata.interfaces.ITransactionAttribute;
import org.ow2.util.deployment.metadata.structures.JMethod;

/**
 * This class represents the annotation metadata of a method.
 * @author Florent Benoit
 */
public class EjbJarMethodMetadata extends CommonMethodMetadata<EjbJarClassMetadata, EjbJarMethodMetadata, EjbJarFieldMetadata>
        implements Cloneable, ITransactionAttribute, IEJBInterceptors, IAnnotationSecurityRolesAllowed,
        IAnnotationSecurityPermitAll, IInterceptorExcludeDefaultInterceptors {

    /**
     * Logger.
     */
    //private static Log logger = LogFactory.getLog(MethodAnnotationMetadata.class);

    /**
     * Type of transaction.
     */
    private TransactionAttributeType transactionAttributeType = null;

    /**
     * Level of transaction (when overriding with XML, need to know if the level is sufficient or not).
     */
    private TransactionAttributeLevel transactionAttributeLevel = TransactionAttributeLevel.ANNOTATION;

    /**
     * &#64;{@link javax.ejb.Remove} annotation.
     */
    private Remove remove = null;

    /**
     * This method is a business method ?
     */
    private boolean businessMethod = false;

    /**
     * PostConstruct method ?
     */
    private boolean postConstruct = false;

    /**
     * PreDestroy method ?
     */
    private boolean preDestroy = false;

    /**
     * PostActivate method ?
     */
    private boolean postActivate = false;

    /**
     * PrePassivate method ?
     */
    private boolean prePassivate = false;

    /**
     * &#64;{@link javax.interceptor.AroundInvoke} method used by interceptors ?
     */
    private boolean aroundInvoke = false;

    /**
     * &#64;{@link javax.interceptor.ExcludeClassInterceptors} method ?
     */
    private boolean excludeClassInterceptors = false;

    /**
     * &#64;{@link javax.ejb.Timeout} method ?
     */
    private boolean timeout = false;

    /**
     * EasyBeans global interceptors.<br>
     * These interceptors correspond to a list of interceptors
     * that need to be present first on this current method.
     */
    private List<JClassInterceptor> globalEasyBeansInterceptors = null;

    /**
     * EasyBeans method interceptors. These interceptors correspond to a list of Interceptors like security or transaction.
     */
    private List<JClassInterceptor> interceptors = null;

    /**
     * List of annotation interceptors.
     */
    private JInterceptors annotationInterceptors = null;

    /**
     * User interceptors. These interceptors correspond to a list of Interceptor that user has specified in its bean class.
     * Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     */
    private Map<InterceptorType, List<JClassInterceptor>> userInterceptors = null;

    /**
     * List of roles that are allowed on this class/bean.
     */
    private List<String> rolesAllowed = null;

    /**
     * This class/bean has the PermitAll annotation.
     */
    private boolean permitAll = false;

    /**
     * This class/bean has the DenyAll annotation.
     */
    private boolean denyAll = false;

    /**
     * &#64;{@link javax.interceptor.ExcludeDefaultInterceptors} method ?
     */
    private boolean excludeDefaultInterceptors = false;

    /**
     * Constructor.
     * @param jMethod the method on which we will set/add metadata
     * @param classAnnotationMetadata the parent metadata.
     */
    public EjbJarMethodMetadata(final JMethod jMethod, final EjbJarClassMetadata classAnnotationMetadata) {
        super(jMethod, classAnnotationMetadata);
    }

    /**
     * @return name of the method
     */
    public String getMethodName() {
        return getJMethod().getName();
    }

    /**
     * @return transaction Attribute type.
     * @see javax.ejb.TransactionAttributeType
     */
    public TransactionAttributeType getTransactionAttributeType() {
        return transactionAttributeType;
    }

    /**
     * Sets Transaction Attribute Level.
     * @param transactionAttributeLevel the level of transaction.
     */
    public void setTransactionAttributeLevel(final TransactionAttributeLevel transactionAttributeLevel) {
        this.transactionAttributeLevel = transactionAttributeLevel;
    }

    /**
     * @return transaction Attribute Level.
     */
    public TransactionAttributeLevel getTransactionAttributeLevel() {
        return transactionAttributeLevel;
    }

    /**
     * Sets Transaction Attribute Type.
     * @see javax.ejb.TransactionAttributeType
     * @param transactionAttributeType the type of transaction.
     */
    public void setTransactionAttributeType(final TransactionAttributeType transactionAttributeType) {
        this.transactionAttributeType = transactionAttributeType;
    }

    /**
     * @return true if the method is a business method.
     */
    public boolean isBusinessMethod() {
        return businessMethod;
    }

    /**
     * This method is a business method.
     * @param flag true/false if method is a business method.
     */
    public void setBusinessMethod(final boolean flag) {
        this.businessMethod = flag;
    }

    /**
     * @return true if the method is a lifecycle method.
     */
    public boolean isLifeCycleMethod() {
        return  isPostActivate() || isPostConstruct() || isPreDestroy() || isPrePassivate();
    }

    /**
     * @return remove attributes  &#64;{@link javax.ejb.Remove} attributes
     */
    public Remove getJRemove() {
        return this.remove;
    }

    /**
     * Sets &#64;{@link javax.ejb.Remove} attribute.
     * @param remove contains the attribute with retainIfException.
     */
    public void setRemove(final Remove remove) {
        this.remove = remove;
    }



    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        String titleIndent = " ";

        // classname
        sb.append(titleIndent);
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));
        sb.append("[\n");

        // Add super class toString()
        sb.append(super.toString());

        // Method
        concatStringBuilder("jMethod", getJMethod(), sb);

        // inherited
        concatStringBuilder("inherited", Boolean.valueOf(isInherited()), sb);

        // transactionAttributeType
        concatStringBuilder("transactionAttributeType", transactionAttributeType, sb);

        // transactionAttributeLevel
        concatStringBuilder("transactionAttributeLevel", transactionAttributeLevel, sb);

        // remove
        concatStringBuilder("remove", remove, sb);

        // businessMethod
        concatStringBuilder("businessMethod", Boolean.valueOf(businessMethod), sb);

        // aroundInvoke
        concatStringBuilder("aroundInvoke", Boolean.valueOf(aroundInvoke), sb);

        // postConstruct
        concatStringBuilder("postConstruct", Boolean.valueOf(postConstruct), sb);

        // preDestroy
        concatStringBuilder("preDestroy", Boolean.valueOf(preDestroy), sb);

        // postActivate
        concatStringBuilder("postActivate", Boolean.valueOf(postActivate), sb);

        // prePassivate
        concatStringBuilder("prePassivate", Boolean.valueOf(prePassivate), sb);

        // timeout
        concatStringBuilder("timeout", Boolean.valueOf(timeout), sb);

        // annotation Interceptors
        concatStringBuilder("annotationInterceptors", annotationInterceptors, sb);

        // interceptors
        concatStringBuilder("interceptors", interceptors, sb);

        // rolesAllowed
        concatStringBuilder("rolesAllowed", rolesAllowed, sb);

        // permitAll
        concatStringBuilder("permitAll", Boolean.valueOf(permitAll), sb);

        // denyAll
        concatStringBuilder("denyAll", Boolean.valueOf(denyAll), sb);

        // denyAll
        concatStringBuilder("excludeDefaultInterceptors", Boolean.valueOf(excludeDefaultInterceptors), sb);

        sb.append(titleIndent);
        sb.append("]\n");
        return sb.toString();
    }

    /**
     * @return true if method has  &#64;{@link javax.ejb.PostActivate}
     */
    public boolean isPostActivate() {
        return postActivate;
    }

    /**
     * Sets true if method has &#64;{@link javax.ejb.PostActivate}.
     * @param postActivate true/false.
     */
    public void setPostActivate(final boolean postActivate) {
        this.postActivate = postActivate;
    }

    /**
     * @return true if method has &#64;{@link javax.annotation.PostConstruct}.
     */
    public boolean isPostConstruct() {
        return postConstruct;
    }

    /**
     * Sets true if method has &#64;{@link javax.annotation.PostConstruct}.
     * @param postConstruct true/false.
     */
    public void setPostConstruct(final boolean postConstruct) {
        this.postConstruct = postConstruct;
    }

    /**
     * @return true if method has &#64;{@link javax.annotation.PreDestroy}.
     */

    public boolean isPreDestroy() {
        return preDestroy;
    }

    /**
     * Sets true if method has &#64;{@link javax.annotation.PreDestroy}.
     * @param preDestroy true/false.
     */
    public void setPreDestroy(final boolean preDestroy) {
        this.preDestroy = preDestroy;
    }

    /**
     * @return true if method has &#64;{@link javax.ejb.PrePassivate}.
     */

    public boolean isPrePassivate() {
        return prePassivate;
    }

    /**
     * Sets true if method has &#64;{@link javax.ejb.PrePassivate}.
     * @param prePassivate true/false.
     */
    public void setPrePassivate(final boolean prePassivate) {
        this.prePassivate = prePassivate;
    }

    /**
     * @return true if method has &#64;{@link javax.ejb.Timeout}.
     */
    public boolean isTimeout() {
        return timeout;
    }

    /**
     * Sets true if method has &#64;{@link javax.ejb.Timeout}.
     * @param timeout true/false.
     */
    public void setTimeout(final boolean timeout) {
        this.timeout = timeout;
    }

    /**
     * @return true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     */
    public boolean isAroundInvoke() {
        return aroundInvoke;
    }

    /**
     * Sets true if method has &#64;{@link javax.interceptor.AroundInvoke}.
     * @param aroundInvoke true/false
     */
    public void setAroundInvoke(final boolean aroundInvoke) {
        this.aroundInvoke = aroundInvoke;
    }

    /**
     * @return true if this method won't use user interceptors.
     */
    public boolean isExcludedClassInterceptors() {
        return excludeClassInterceptors;
    }

    /**
     * Flag this method as a method which exclude user interceptors.
     * @param excludeClassInterceptors true if this method is a method which exclude user interceptors.
     */
    public void setExcludeClassInterceptors(final boolean excludeClassInterceptors) {
        this.excludeClassInterceptors = excludeClassInterceptors;
    }

    /**
     * @return list of interceptors that enhancer will use. (ie : security/transaction)
     */
    public List<JClassInterceptor> getInterceptors() {
        return interceptors;
    }

    /**
     * Sets the list of interceptors(tx, security, etc) that enhancers will use.<br>
     * These interceptors are defined per methods.
     * @param interceptors list of interceptors that enhancer will use.
     */
    public void setInterceptors(final List<JClassInterceptor> interceptors) {
        this.interceptors = interceptors;
    }

    /**
     * @return object representing list of &#64;{@link javax.interceptor.Interceptors}.
     */
    public JInterceptors getAnnotationInterceptors() {
        return annotationInterceptors;
    }

    /**
     * Sets the object representing the &#64;{@link javax.interceptor.Interceptors} annotation.
     * @param annotationInterceptors list of classes
     */
    public void setAnnotationsInterceptors(final JInterceptors annotationInterceptors) {
        this.annotationInterceptors = annotationInterceptors;
    }

    /**
     * @return Map&lt;interceptor type &lt;--&gt; List of methods/class corresponding to the interceptor&gt;
     * of user interceptors that enhancer will use.
     */
    public Map<InterceptorType, List<JClassInterceptor>> getUserEasyBeansInterceptors() {
        return userInterceptors;
    }

    /**
     * Sets the list of user interceptors that enhancers will use.<br>
     * These interceptors are defined in bean class.
     * @param userInterceptors list of interceptors that enhancer will use.
     */
    public void setUserInterceptors(final Map<InterceptorType, List<JClassInterceptor>> userInterceptors) {
        this.userInterceptors = userInterceptors;
    }

    /**
     * @return list of global interceptors that enhancer will use. (ie : Remove interceptor)
     */
    public List<JClassInterceptor> getGlobalEasyBeansInterceptors() {
        return globalEasyBeansInterceptors;
    }

    /**
     * Sets the list of global interceptors that enhancers will use.
     * @param globalEasyBeansInterceptors list of interceptors that enhancer will use.
     */
    public void setGlobalEasyBeansInterceptors(final List<JClassInterceptor> globalEasyBeansInterceptors) {
        this.globalEasyBeansInterceptors = globalEasyBeansInterceptors;
    }

    /**
     * @return a clone object.
     */
    @Override
    public Object clone() {
        EjbJarMethodMetadata newMethodAnnotationMetadata = new EjbJarMethodMetadata(getJMethod(), getClassMetadata());
        newMethodAnnotationMetadata.setAnnotationsInterceptors(annotationInterceptors);
        newMethodAnnotationMetadata.setAroundInvoke(aroundInvoke);
        newMethodAnnotationMetadata.setBusinessMethod(businessMethod);
        newMethodAnnotationMetadata.setExcludeClassInterceptors(excludeClassInterceptors);
        newMethodAnnotationMetadata.setInherited(isInherited(), getOriginalClassMetadata());
        newMethodAnnotationMetadata.setInterceptors(interceptors);
        newMethodAnnotationMetadata.setJAnnotationResource(getJAnnotationResource());
        newMethodAnnotationMetadata.setJEjbEJB(getJEjbEJB());
        newMethodAnnotationMetadata.setJavaxPersistenceContext(getJavaxPersistenceContext());
        newMethodAnnotationMetadata.setJavaxPersistenceUnit(getJavaxPersistenceUnit());
        newMethodAnnotationMetadata.setPostActivate(postActivate);
        newMethodAnnotationMetadata.setPostConstruct(postConstruct);
        newMethodAnnotationMetadata.setPreDestroy(preDestroy);
        newMethodAnnotationMetadata.setPrePassivate(prePassivate);
        newMethodAnnotationMetadata.setRemove(remove);
        newMethodAnnotationMetadata.setTimeout(timeout);
        newMethodAnnotationMetadata.setTransactionAttributeType(transactionAttributeType);
        newMethodAnnotationMetadata.setTransactionAttributeLevel(transactionAttributeLevel);
        newMethodAnnotationMetadata.setUserInterceptors(userInterceptors);
        newMethodAnnotationMetadata.setGlobalEasyBeansInterceptors(globalEasyBeansInterceptors);
        newMethodAnnotationMetadata.setDenyAll(denyAll);
        newMethodAnnotationMetadata.setPermitAll(permitAll);
        newMethodAnnotationMetadata.setRolesAllowed(rolesAllowed);
        return newMethodAnnotationMetadata;
    }


    /**
     * Set the list of roles allowed on this class/method.
     * @param rolesAllowed the list of roles.
     */
    public void setRolesAllowed(final List<String> rolesAllowed) {
        this.rolesAllowed = rolesAllowed;
    }

    /**
     * @return the list of roles allowed on this class/method.
     */
    public List<String> getRolesAllowed() {
        return rolesAllowed;
    }

    /**
     * This method has PermitAll annotation.
     * @param permitAll the boolean value.
     */
    public void setPermitAll(final boolean permitAll) {
        this.permitAll = permitAll;
    }

    /**
     * @return true if PermitAll annotation.
     */
    public boolean hasPermitAll() {
        return permitAll;
    }

    /**
     * This method has DenyAll annotation.
     * @param denyAll the boolean value.
     */
    public void setDenyAll(final boolean denyAll) {
        this.denyAll = denyAll;
    }

    /**
     * @return true if DenyAll annotation.
     */
    public boolean hasDenyAll() {
        return denyAll;
    }

    /**
     * @return true if this class won't use default interceptors.
     */
    public boolean isExcludedDefaultInterceptors() {
        return excludeDefaultInterceptors;
    }

    /**
     * Flag this class as a class which exclude default interceptors.
     * @param excludeDefaultInterceptors true if this class is a class which exclude default interceptors.
     */
    public void setExcludeDefaultInterceptors(final boolean excludeDefaultInterceptors) {
        this.excludeDefaultInterceptors = excludeDefaultInterceptors;
    }

}
