/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: WARDeploymentDesc.java 2059 2007-11-22 17:22:33Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.deployment.xml;

import java.net.URL;
import java.util.Iterator;

import org.ow2.easybeans.deployment.xml.parsing.ParsingException;
import org.ow2.easybeans.deployment.xml.parsing.WARDeploymentDescLoader;
import org.ow2.easybeans.deployment.xml.parsing.WARTLDLoader;
import org.ow2.easybeans.deployment.xml.struct.WAR;
import org.ow2.util.ee.deploy.api.archive.ArchiveException;
import org.ow2.util.ee.deploy.api.archive.IArchive;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Utility class to get a WAR deployment descriptor.
 * @author Florent Benoit
 */
public final class WARDeploymentDesc {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(WARDeploymentDesc.class);

    /**
     * Utility class, no public constructor.
     */
    private WARDeploymentDesc() {

    }

    /**
     * Gets WAR for a given archive.
     * @param archive file/directory.
     * @throws WARDeploymentDescException if no xml is present.
     * @return WAR instance.
     */
    public static WAR getWAR(final IArchive archive) throws WARDeploymentDescException {

        // Get XML entry
        URL webXmlURL = null;
        try {
            webXmlURL = archive.getResource("WEB-INF/web.xml");
        } catch (ArchiveException e) {
            throw new WARDeploymentDescException("Cannot get resource 'WEB-INF/web.xml' on the archive '"
                    + archive.getName() + "'.");
        }

        WAR war = null;
        // Get WAR object (if url valid)
        if (webXmlURL != null) {
             try {
                 war = WARDeploymentDescLoader.loadDeploymentDescriptor(webXmlURL);
            } catch (ParsingException e) {
                throw new WARDeploymentDescException("Cannot parse URL '" + webXmlURL + "'.", e);
            }
        } else {
            war = new WAR();
        }

        // Check if there are .tld files
        Iterator<URL> itUrlResources = null;
        try {
            itUrlResources = archive.getResources();
        } catch (ArchiveException e) {
            throw new WARDeploymentDescException("Cannot get resources on the archive '"
                    + archive.getName() + "'.");
        }

        // Scan all URLs to search .tld
        while (itUrlResources.hasNext()) {
            URL url = itUrlResources.next();
            if (url.getPath().endsWith(".tld")) {
                // Complete the structure of the war object.
                try {
                    WARTLDLoader.scanTLD(url, war);
                } catch (ParsingException e) {
                    logger.error("Cannot analyze the TLD file ''{0}''", url, e);
                }
            }
        }



        return war;
    }
}
