/**
 * EasyBeans
 * Copyright (C) 2006-2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: EasyBeansContextListener.java 2584 2008-03-07 11:00:56Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.server.war;

import static org.ow2.easybeans.util.url.URLUtils.fileToURL2;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.ow2.easybeans.deployable.DeployerFactory;
import org.ow2.easybeans.deployer.JettyDeployer;
import org.ow2.easybeans.deployer.TomcatDeployer;
import org.ow2.easybeans.server.EasyBeans;
import org.ow2.easybeans.server.Embedded;
import org.ow2.easybeans.server.EmbeddedConfigurator;
import org.ow2.easybeans.server.EmbeddedException;
import org.ow2.easybeans.server.ServerConfig;
import org.ow2.easybeans.util.url.URLUtilsException;
import org.ow2.util.ee.deploy.api.deployer.DeployerException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Listener class which is notified of lifecycle events. It allows to start
 * EasyBeans at startup and stop it at the end.
 * @author Florent Benoit
 */
public class EasyBeansContextListener implements ServletContextListener {

    /**
     * Resource containing the list of files.
     */
    private static final String COMMON_LIBRARIES_LIST = "easybeans-exported-libraries.lst";

    /**
     * Default XML file.
     */
    public static final String DEFAULT_XML_FILE = "org/ow2/easybeans/server/war/easybeans-default.xml";

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(EasyBeansContextListener.class);

    /**
     * Embedded instance.
     */
    private Embedded embedded = null;

    /**
     * List of available supported container.
     */
    private enum AvailableWebContainer {
        /**
         * Containers supported.
         */
        TOMCAT, JETTY, UNKNOWN
    }

    /**
     * Container detected ?.
     */
    private AvailableWebContainer containerType;

    /**
     * Notification that the web application initialization process is starting.
     * All ServletContextListeners are notified of context initialization before
     * any filter or servlet in the web application is initialized.
     * @param servletContextEvent event class for notifications about changes to
     *        the servlet context of a web application.
     */
    public void contextInitialized(final ServletContextEvent servletContextEvent) {

        // Create and configure EasyBeans embedded server
        if (this.embedded == null) {

            // user configuration ?
            URL xmlConfigurationURL = Thread.currentThread().getContextClassLoader().getResource(EasyBeans.USER_XML_FILE);

            if (xmlConfigurationURL == null) {
                logger.warn("No {0} resource found in classpath, use default settings", EasyBeans.USER_XML_FILE);
                xmlConfigurationURL = Thread.currentThread().getContextClassLoader().getResource(DEFAULT_XML_FILE);
            }
            try {
                this.embedded = EmbeddedConfigurator.create(xmlConfigurationURL);
            } catch (EmbeddedException e) {
                throw new IllegalStateException("Cannot create the embedded server", e);
            }

            // Detects Tomcat/Jetty/etc.
            String ejb3ParentDir = detectWebContainerDirectory();
            logger.info("Detecting '" + this.containerType + "' web container");

            // Create ejb3s directory
            File workDir = new File(ejb3ParentDir + File.separator + Embedded.DEFAULT_DEPLOY_DIRECTORY);
            workDir.mkdir();

            logger.info("Add deploy directory ''{0}''", workDir);
            getServerConfig().addDeployDirectory(workDir);

            // Register the deployers
            switch (this.containerType) {
            // Use Tomcat deployer if it is Tomcat
            case TOMCAT:
                DeployerFactory.setClassName(TomcatDeployer.class.getName());
                break;
            case JETTY:
                DeployerFactory.setClassName(JettyDeployer.class.getName());
                try {
                    JettyDeployer.setContextEvent(servletContextEvent);
                } catch (DeployerException e) {
                    logger.warn("Unable to set the servlet context event on the jetty deployer."
                            + "The default deployer will be used.", e);
                    // reset the deployer
                    DeployerFactory.setClassName(null);
                }
                break;
            case UNKNOWN:
            default:
                logger.info("Using default deployer as the web container has not be detected");
                break;

            }

            registerLibraries(servletContextEvent);
            try {
                this.embedded.start();
            } catch (EmbeddedException e) {
                throw new IllegalStateException("Cannot start embedded EasyBeans server", e);
            }
        }
    }

    /**
     * Notification that the servlet context is about to be shut down. All
     * servlets and filters have been destroy()ed before any
     * ServletContextListeners are notified of context destruction.
     * @param servletContextEvent event class for notifications about changes to
     *        the servlet context of a web application.
     */
    public void contextDestroyed(final ServletContextEvent servletContextEvent) {
        if (this.embedded != null) {
            logger.info("Stopping EasyBeans embedded server...");
            try {
                this.embedded.stop();
            } catch (EmbeddedException e) {
                throw new IllegalStateException("Cannot stop the embedded server", e);
            } catch (Throwable e) {
                logger.error("Unexpected error when stopping the embedded server", e);
                throw new IllegalStateException("Unexpected error when stopping the embedded server", e);
            } finally {
                this.embedded = null;
            }
        }
    }

    /**
     * Gets the directory for EJB3 depending on the servlet container.
     * @return path to catalina.base/jetty.home/etc.
     */
    private String detectWebContainerDirectory() {
        // catalina base ?
        String catalinaBase = System.getProperty("catalina.base");
        if (catalinaBase != null) {
            this.containerType = AvailableWebContainer.TOMCAT;
            return catalinaBase;
        }

        // Jetty ?
        String jettyHome = System.getProperty("jetty.home");
        if (jettyHome != null) {
            this.containerType = AvailableWebContainer.JETTY;
            return jettyHome;
        }

        // not found, return tmp directory
        this.containerType = AvailableWebContainer.UNKNOWN;
        return System.getProperty("java.io.tmpdir");
    }

    /**
     * Register the EasyBeans libraries in the root classloader (allowing each
     * applications to communicate with EasyBeans).
     * @param servletContextEvent event class for notifications about changes to
     *        the servlet context of a web application.
     */
    private void registerLibraries(final ServletContextEvent servletContextEvent) {
        if (this.containerType.equals(AvailableWebContainer.TOMCAT)) {
            registerTomcat(servletContextEvent);
        } else if (this.containerType.equals(AvailableWebContainer.JETTY)) {
            registerJetty(servletContextEvent);
        }

    }

    /**
     * Register the EasyBeans libraries for Jetty.
     * @param servletContextEvent event class for notifications about changes to
     *        the servlet context of a web application.
     */
    private void registerJetty(final ServletContextEvent servletContextEvent) {
        // Do nothing with Jetty

        // Current Classloader
        //ClassLoader currentCL = Thread.currentThread().getContextClassLoader();

        // Get parent
        //ClassLoader cl = currentCL.getParent();

        // Add libraries
        //addLib(cl, servletContextEvent);
    }

    /**
     * Register the EasyBeans libraries for Tomcat.
     * @param servletContextEvent event class for notifications about changes to
     *        the servlet context of a web application.
     */
    private void registerTomcat(final ServletContextEvent servletContextEvent) {

        // Current Classloader
        ClassLoader currentCL = Thread.currentThread().getContextClassLoader();

        // Now, search shared classloader (child of server CL)
        // Assume that it is a correct Tomcat installation

        // For tomcat 6.x, this is only the direct parent while for Tomcat
        // 5.5.x, it's parent of parent
        ClassLoader cl = currentCL.getParent();
        ClassLoader parent2 = cl.getParent();
        if ("org.apache.catalina.loader.StandardClassLoader".equals(parent2.getClass().getName())) {
            // Tomcat 5.5.x case, gets the parent again
            cl = parent2;
        }
        // Check class :
        if (!"org.apache.catalina.loader.StandardClassLoader".equals(cl.getClass().getName())) {
            throw new IllegalStateException("Didn't find common classloader");
        }

        // Add libraries
        addLib(cl, servletContextEvent);

    }

    /**
     * Add libraries in the given classloader.
     * @param cl the classloader where to add jars (which needs to be an
     *        URLClassLoader)
     * @param servletContextEvent the servlet context event
     */
    protected void addLib(final ClassLoader cl, final ServletContextEvent servletContextEvent) {
        // Get URL ClassLoader
        URLClassLoader urlCl = null;
        if (cl instanceof URLClassLoader) {
            urlCl = (URLClassLoader) cl;
        } else {
            throw new IllegalStateException("Parent classloader is not an URL classloader. Found '" + cl + "'.");
        }

        // Get addURL method
        Method addURL = null;
        try {
            addURL = URLClassLoader.class.getDeclaredMethod("addURL", new Class[] {URL.class});
        } catch (SecurityException e) {
            throw new IllegalArgumentException("Cannot get addURL method on the class '" + URLClassLoader.class + "'.", e);
        } catch (NoSuchMethodException e) {
            throw new IllegalArgumentException("Cannot get addURL method on the class '" + URLClassLoader.class + "'.", e);
        }
        addURL.setAccessible(true);

        // Add libraries
        ServletContext sContext = servletContextEvent.getServletContext();
        List<String> libraries = new ArrayList<String>();
        getLibraries(sContext, libraries);

        for (String library : libraries) {
            URL url = null;
            try {
                url = fileToURL2(new File(library));
            } catch (URLUtilsException e) {
                throw new IllegalArgumentException("Cannot get URL on file '" + library + "'.", e);
            }
            try {
                logger.debug("Registering '" + url + "'.");
                addURL.invoke(urlCl, url);
            } catch (IllegalArgumentException e) {
                throw new IllegalStateException("Cannot register the URL '" + url + "'.", e);
            } catch (IllegalAccessException e) {
                throw new IllegalStateException("Cannot register the URL '" + url + "'.", e);
            } catch (InvocationTargetException e) {
                throw new IllegalStateException("Cannot register the URL '" + url + "'.", e);
            }
        }
        addURL.setAccessible(false);
    }

    /**
     * Retrieve a List of libraries to be loaded by TopLevel ClassLoaders.
     * @param sContext ServletContext.
     * @param libraries List of libs.
     */
    protected void getLibraries(final ServletContext sContext, final List<String> libraries) {
        // Files that needs to be in the commons classloader are stored in a
        // properties file.

        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();

        List<String> librariesList = getExportedLibraries();

        for (String library : librariesList) {

            URL url = classLoader.getResource(library);
            if (url == null) {
                throw new IllegalStateException("No resource named '" + library
                        + "' was found in the current classloader.");
            }

            // got url, open it
            URLConnection urlConnection = null;
            try {
                urlConnection = url.openConnection();
            } catch (IOException e) {
                throw new IllegalStateException("Cannot open connection on the URL '" + url + "'.", e);
            }
            urlConnection.setDefaultUseCaches(false);

            Reader reader = null;
            try {
                try {
                    reader = new InputStreamReader(urlConnection.getInputStream());
                } catch (IOException e) {
                    throw new IllegalStateException("Cannot get inputstream on the URL '" + url + "'.", e);
                }
                BufferedReader bufferedReader = new BufferedReader(reader);
                String fileName = null;
                try {
                    while ((fileName = bufferedReader.readLine()) != null) {
                        fileName = fileName.trim();
                        // only lines with values
                        if (fileName.length() >= 0) {
                            // Get filename
                            libraries.add(sContext.getRealPath("/WEB-INF/lib/" + fileName.trim()));
                        }
                    }
                } catch (IOException e) {
                    throw new IllegalStateException("Cannot read the inputstream on the URL '" + url + "'.", e);
                } finally {
                    if (bufferedReader != null) {
                        try {
                            bufferedReader.close();
                        } catch (IOException e) {
                            throw new IllegalStateException("Cannot close reader object on the URL '" + url + "'.", e);
                        }
                    }
                }
            } finally {
                if (reader != null) {
                    try {
                        reader.close();
                    } catch (IOException e) {
                        throw new IllegalStateException("Cannot close reader object on the URL '" + url + "'.", e);
                    }
                }
            }
        }

    }

    /**
     * @return a List of resources that contains the libraries to be exported.
     */
    protected List<String> getExportedLibraries() {
        List<String> list = new ArrayList<String>();
        list.add(COMMON_LIBRARIES_LIST);
        return list;
    }

    /**
     * @return returns the Easybeans ServerConfig object.
     */
    protected ServerConfig getServerConfig() {
        return this.embedded.getServerConfig();
    }
}
