/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: FileUtils.java 1970 2007-10-16 11:49:25Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.util.files;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.channels.FileChannel;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This class manages operation done many times on files, like copying them.
 * @author Florent Benoit
 */
public final class FileUtils {

    /**
     * Size of the buffer.
     */
    private static final int BUFFER_SIZE = 2048;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(FileUtils.class);

    /**
     * Utility class, no public constructor.
     */
    private FileUtils() {

    }

    /**
     * Unpack the source archive in a given directory and returns directory
     * directory created.
     * @param packedJar source JarFile to be unpacked
     * @param dest the destination folder
     * @throws FileUtilsException When unpack fails
     */
    public static void unpack(final JarFile packedJar, final File dest) throws FileUtilsException {

        JarEntry entry = null;

        // get entries of the jar file
        Enumeration entries = packedJar.entries();
        while (entries.hasMoreElements()) {
            entry = (JarEntry) entries.nextElement();

            // File entry
            File entryFile = new File(dest, entry.getName());

            // Create directory
            if (entry.isDirectory()) {
                if (!entryFile.exists()) {
                    // create parent directories (with mkdirs)
                    if (!entryFile.mkdirs()) {
                        String err = "Can not create directory " + entryFile + ", Check the write access.";
                        throw new FileUtilsException(err);
                    }
                }
                continue;
            }

            // If it's a file, we must extract the file
            // Ensure that the directory exists.
            entryFile.getParentFile().mkdirs();

            InputStream is = null;
            // get the input stream
            try {
                is = packedJar.getInputStream(entry);
                // Dump to the file
                dump(is, entryFile);
            } catch (IOException ioe) {
                throw new FileUtilsException("Cannot get inputstream of entry '" + entry + "' of file '" + packedJar + "'.");
            } finally {
                try {
                    is.close();
                } catch (IOException ioe) {
                    logger.debug("Cannot close input stream", ioe);
                }
            }

        }
    }

    /**
     * Write the given input stream in the given file.
     * @param in the inputStream to copy.
     * @param entryFile the file where the inputStream must be dumped.
     * @throws FileUtilsException if the dump failed.
     */
    public static void dump(final InputStream in, final File entryFile) throws FileUtilsException {

        try {
            // File output
            FileOutputStream out = new FileOutputStream(entryFile);
            int n = 0;
            try {
                // buffer
                byte[] buffer = new byte[BUFFER_SIZE];
                n = in.read(buffer);
                while (n > 0) {
                    out.write(buffer, 0, n);
                    n = in.read(buffer);
                }
            } finally {
                out.close();
            }
        } catch (IOException e) {
            String err = "Error while unpacking entry " + entryFile + " : ";
            throw new FileUtilsException(err, e);
        }
    }

    /**
     * Copy a file.
     * @param src source file
     * @param dest dest file
     * @throws FileUtilsException if the copy of the file failed
     */
    public static void copyFile(final String src, final String dest) throws FileUtilsException {
        copyFile(new File(src), new File(dest));
    }

    /**
     * Copy a file.
     * @param src source file
     * @param dest dest file
     * @throws FileUtilsException if the copy of the file failed
     */
    public static void copyFile(final File src, final File dest) throws FileUtilsException {
        File newDest = null;
        if (dest.isDirectory()) {
            logger.debug("Copy a file to a directory, append source filename to directory.");
            newDest = new File(dest, src.getName());
        } else {
            newDest = dest;
        }

        FileInputStream fIn = null;
        FileOutputStream fOut = null;
        FileChannel fcIn = null;
        try {

            // InputStream
            fIn = new FileInputStream(src);
            fOut = new FileOutputStream(newDest);

            // nio channel
            FileChannel sourceFC = fIn.getChannel();
            FileChannel targetFC = fOut.getChannel();

            targetFC.transferFrom(sourceFC, 0, sourceFC.size());
        } catch (Exception e) {
            throw new FileUtilsException("Error during copy file : " + src + " -> " + dest, e);
        } finally {
            try {
                fOut.close();
                fIn.close();
                fcIn.close();
            } catch (Exception e) {
                logger.debug("Cannot close some i/o which are open.", e);
            }

        }
    }

    /**
     * @param path file/directory to be deleted
     * @return true if deletion was OK
     */
    public static boolean delete(final String path) {
        return delete(new File(path));
    }

    /**
     * @param f file/directory to be deleted
     * @return true if deletion was OK
     */
    public static boolean delete(final File f) {
        if (f.isFile()) {
            return f.delete();
        }
        // else (if directory)
        File[] children = f.listFiles();
        if (children == null) {
            // no children
            return f.delete();
        }

        // There are children
        boolean result = true;
        for (int i = 0; i < children.length; i++) {
            result &= delete(children[i]);
        }
        return result && f.delete();
    }

    /**
     * Copy a directory recursively.
     * @param src source directory
     * @param dest dest directory
     * @throws FileUtilsException if the copy of the directory failed
     */
    public static void copyDirectory(final String src, final String dest) throws FileUtilsException {
        copyDirectory(new File(src), new File(dest));
    }

    /**
     * Copy a directory recursively.
     * @param src source directory
     * @param dest dest directory
     * @throws FileUtilsException if the copy of the directory failed
     */
    public static void copyDirectory(final File src, final File dest) throws FileUtilsException {

        if (!src.isDirectory()) {
            // We don not accept file arguments !
            throw new IllegalArgumentException("Source '" + src + "' must be a directory");
        }

        // create the destination directory if it is inexistant
        if (!dest.exists()) {
            dest.mkdirs();
        }

        // copy the files of the source directory
        File[] childs = src.listFiles();
        if (childs != null) {
            // childs
            for (int i = 0; i < childs.length; i++) {
                File child = childs[i];
                if (child.isFile()) {
                    // file
                    copyFile(child, dest);
                } else {
                    // directory
                    copyDirectory(child, new File(dest, child.getName()));
                }
            }
        }
    }

}
