/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CtxClassLoaderObjectInputStream.java 1970 2007-10-16 11:49:25Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.util.marshalling;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.lang.reflect.Proxy;

/**
 * Reader which set the context classloader when recreating the object.
 * @author Florent Benoit
 */
public class CtxClassLoaderObjectInputStream extends ObjectInputStream {

    /**
     * This inputstream reader needs an input stream for its constructor.
     * @param inputStream input stream containing bytes
     * @throws IOException if super class fails
     */
    public CtxClassLoaderObjectInputStream(final InputStream inputStream) throws IOException {
        super(inputStream);
    }

    /**
     * Load the local class equivalent of the specified stream class
     * description.
     * @param desc an instance of class <code>ObjectStreamClass</code>
     * @return a <code>Class</code> object corresponding to <code>desc</code>
     * @throws IOException any of the usual input/output exceptions
     * @throws ClassNotFoundException if class of a serialized object cannot be
     *         found
     */
    @Override
    protected Class<?> resolveClass(final ObjectStreamClass desc) throws IOException, ClassNotFoundException {
        // Use the thread classloader
        return Class.forName(desc.getName(), false, Thread.currentThread().getContextClassLoader());
    }

    /**
     * Returns a proxy class that implements the interfaces named in a proxy
     * class descriptor; subclasses may implement this method to read custom
     * data from the stream along with the descriptors for dynamic proxy
     * classes, allowing them to use an alternate loading mechanism for the
     * interfaces and the proxy class.
     * @param interfacesName the list of interface names that were deserialized in
     *        the proxy class descriptor
     * @return a proxy class for the specified interfaces
     * @throws IOException any exception thrown by the underlying
     *         <code>InputStream</code>
     * @throws ClassNotFoundException if the proxy class or any of the named
     *         interfaces could not be found
     * @see java.io.ObjectOutputStream#annotateProxyClass(Class)
     * @since 1.3
     */
    @Override
    protected Class<?> resolveProxyClass(final String[] interfacesName) throws IOException, ClassNotFoundException {
        // Load the interfaces with the
        Class[] interfaces = new Class[interfacesName.length];
        for (int i = 0; i < interfaces.length; i++) {
            interfaces[i] = Class.forName(interfacesName[i], false, Thread.currentThread().getContextClassLoader());
        }

        // Use the thread classloader to make the proxy
        try {
            return Proxy.getProxyClass(Thread.currentThread().getContextClassLoader(), interfaces);
        } catch (IllegalArgumentException e) {
            throw new ClassNotFoundException(null, e);
        }

    }

}
